/*
 * code visualize atoms
 *
 * author: frank.zirkelbach@physik.uni-augsburg.de
 *
 */

#define _GNU_SOURCE
#include <stdio.h>
//#include <stdlib.h>
//#include <unistd.h>
//#include <string.h>
//#include <sys/types.h>
//#include <sys/stat.h>
//#include <fcntl.h>

#include "moldyn.h"

/* pse */
#define PSE_NAME
#define PSE_COL
#include "pse.h"
#undef PSE_NAME
#undef PSE_COL

typedef struct s_data {
	int ca;
	double radius;
	double lc;
	int ma;
	double ox,oy,oz;
} t_data;

int usage(char *prog) {

	printf("\nusage:\n");
	printf("  %s <file> <centre atom> <radius> <lc> [marked atom]\n\n",
	       prog);

	return -1;
}

int process(t_moldyn *moldyn,t_atom *itom,t_atom *jtom,void *ptr,u8 bc) {

	t_3dvec dist;
	double d;
	t_data *data;

	data=ptr;

	v3_sub(&dist,&(itom->r),&(jtom->r));
	if(bc) check_per_bound(moldyn,&dist);
	d=v3_norm(&dist);

	if(d<=data->radius) {
		printf("%s %f %f %f %s %f\n",
		       pse_name[jtom->element],
		       jtom->r.x+data->ox,jtom->r.y+data->oy,jtom->r.z+data->oz,
		       (jtom->tag==data->ma)?"Red":pse_col[jtom->element],
		       jtom->ekin);
	}

	return 0;
}

int main(int argc,char **argv) {

	t_moldyn moldyn;
	t_data data;
	int ret;
	t_atom *atom;

	if(argc<4) {
		usage(argv[0]);
		return -1;
	}
	
	data.ca=atoi(argv[2]);
	data.radius=atof(argv[3]);
	data.lc=atof(argv[4]);
	data.ma=-1;
	if(argc==6)
		data.ma=atoi(argv[5]);

	memset(&moldyn,0,sizeof(t_moldyn));

	printf("[visual atoms] reading save file ...\n");
	ret=moldyn_read_save_file(&moldyn,argv[1]);
	if(ret) {
		printf("[visual atoms] exit!\n");
		return ret;
	}

	/* link cell init */
	moldyn.cutoff=data.radius;
	link_cell_init(&moldyn,VERBOSE);

	atom=&(moldyn.atom[data.ca]);

	/* prepare offset */
	data.ox=0.0;
	if(atom->r.x<0) {
		while((atom->r.x+data.ox)<(-data.lc/2.0))
			data.ox+=data.lc;
	}
	else {
		while((atom->r.x+data.ox)>(data.lc/2.0))
			data.ox-=data.lc;
	}

	data.oy=0.0;
	if(atom->r.y<0) {
		while((atom->r.y+data.oy)<(-data.lc/2.0))
			data.oy+=data.lc;
	}
	else {
		while((atom->r.y+data.oy)>(data.lc/2.0))
			data.oy-=data.lc;
	}

	data.oz=0.0;
	if(atom->r.z<0) {
		while((atom->r.z+data.oz)<(-data.lc/2.0))
			data.oz+=data.lc;
	}
	else {
		while((atom->r.z+data.oz)>(data.lc/2.0))
			data.oz-=data.lc;
	}

	/* print the centered atom */
	printf("%s %f %f %f %s %f\n",
	       pse_name[atom->element],atom->r.x+data.ox,atom->r.y+data.oy,
               atom->r.z+data.oz,"Green",atom->ekin);

	process_neighbours(&moldyn,&data,atom,process);
	
	link_cell_shutdown(&moldyn);
	moldyn_free_save_file(&moldyn);

	return 0;
}

